#!/bin/bash
. ~/.nvm/nvm.sh
nvm use v0.2.6
node ./index.js
nvm use v0.3.3
node ./index.js
nvm use v0.4.0
node ./index.js
nvm use v0.4.3
node ./index.js
nvm use v0.4.7
node ./index.js
nvm use v0.4.8
node ./index.js
nvm use v0.6.2
node ./index.js
